(function(global) {
  //开启严格模式，规范代码，提高浏览器运行效率
  'use strict';
  (function() {
    function f(n) {
      return n < 10 ? '0' + n : n;
    }
    if (typeof Date.prototype.toJSON !== 'function') {
      Date.prototype.toJSON = function() {
        return isFinite(this.valueOf())
          ? this.getUTCFullYear() +
              '-' +
              f(this.getUTCMonth() + 1) +
              '-' +
              f(this.getUTCDate()) +
              'T' +
              f(this.getUTCHours()) +
              ':' +
              f(this.getUTCMinutes()) +
              ':' +
              f(this.getUTCSeconds()) +
              'Z'
          : null;
      };
    }
    if (typeof Array.prototype.in_array !== 'function') {
      Array.prototype.in_array = function(e) {
        var r = new RegExp(',' + e + ',');
        return r.test(',' + this.join(this.S) + ',');
      };
    }
  })();
  var JOk = function(options) {
    this.debug = options.debug;
    this.jsApiList = options.jsApiList;
    this.nativeMethods = [];
  };
  function jlog(params) {
    if (ok.debug) {
      console.log(new Date().toJSON() + ' ---> ' + params);
    }
  }
  var defaultParams = {
    debug: false,
    appId: '',
    timestamp: '',
    nonceStr: '',
    signature: '',
    jsApiList: []
  };
  function toJson(obj) {
    if (typeof obj == 'string') {
      return JSON.parse(obj);
    } else {
      return obj;
    }
  }
  
  function parseObject(obj) {
    if (!obj) {
      return {};
    }
    if(obj.constructor != Object){
        return {};
    }
    return obj;
  }
  function checkInvoke(method, params, opts) {
    if (ok.nativeMethods.in_array(method)) {
      invoke(method, params, opts);
    } else {
      console.log('permission denied to call method : ' + method);
      console.log('to check api list ');
    }
  }
  function invoke(method, params, opts) {
    global.OkJSBridge
      ? global.OkJSBridge.websiteReq(method, params, function(result) {
          clientReturn(method, result, opts);
        })
      : console.log(method + '->params:' + JSON.stringify(params));
  }
  function clientReturn(interfaceNm, result, a) {
    var jo = toJson(result);
    jlog('JSBridge interfaceNm:' + interfaceNm);
    jlog('JSBridge clientReturn:' + JSON.stringify(jo));
    var msg =  typeof jo.errMsg !== 'undefined' ? jo.errMsg : typeof jo.msg !== 'undefined' ? jo.msg : '';
    var responseStatus = msg.substring(interfaceNm.length + 1);
    switch (responseStatus) {
      case 'success':
        a.success && a.success(jo);
        break;
      case 'fail':
        a.fail && a.fail(jo);
        break;
      default:
        a.fail && a.fail(jo);
    }
    a.complete && a.complete(jo);
  }
  var API = {
    //界面 交互
    showToast: 'Interactivity.showToast',
    showLoading: 'Interactivity.showLoading',
    hideLoading: 'Interactivity.hideLoading',
    showModal: 'Interactivity.showModal',

    //界面 导航栏
    operateClientBottomBar: 'runClientAction.operateClientBottomBar',
    // 获取发送邮件信息
    getEmailFileInfos: "runClientAction.getEmailFileInfos",
    // 跳转进入企业
    jumpToCorp: "runClientAction.jumpToCorp",
    // 消息界面打开同屏会议
    joinScreenShare: "runClientAction.joinScreenShare",
    // 显示vconsole
    showVconsole: 'runClientAction.showVconsole',
    // 开通同屏进入同屏页面
    enterScreenShare: 'runClientAction.enterScreenShare',
    // 调起客户端同屏支付
    payScreenShareOrder: 'runClientAction.payScreenShareOrder',

    showShareMenu: 'Navigation.showShareMenu',
    hideShareMenu: 'Navigation.hideShareMenu',
    showNavigationBar: 'Navigation.showNavigationBar',
    hideNavigationBar: 'Navigation.hideNavigationBar',
    setNavigationBar: 'Navigation.setNavigationBar',
    closeWindow: 'Navigation.closeWindow',
    //设备与系统
    getSystemInfo: 'SystemInfo.getSystemInfo',
    getNetworkType: 'SystemInfo.getNetworkType',
    setClipboardData: 'SystemInfo.setClipboardData',
    getClipboardData: 'SystemInfo.getClipboardData',
    setScreenBrightness: 'SystemInfo.setScreenBrightness',
    getScreenBrightness: 'SystemInfo.getScreenBrightness',
    setKeepScreenOn: 'SystemInfo.setKeepScreenOn',
    vibrateLong: 'SystemInfo.vibrateLong',
    vibrateShort: 'SystemInfo.vibrateShort',
    scanCode: 'SystemInfo.scanCode',
    sendAction: 'SystemInfo.sendAction',
    //第三方
    shareAppMessage: 'Social.shareAppMessage',
    launchMiniProgram: 'Social.launchMiniProgram',
    shareMiniProgramMessage: 'Social.shareMiniProgramMessage',
    startPay: 'Social.startPay',
    //音频
    // startRecord: 'Audio.startRecord',
    // stopRecord: 'Audio.stopRecord',
    //位置
    getLocation: 'Location.getLocation',
    //数据缓存
    setStorage: 'Storage.setStorage',
    getStorage: 'Storage.getStorage',
    //媒体
    chooseImage: 'Media.chooseImage',
    //网络
    // downloadFile: 'Network.downloadFile',
    // uploadFile: 'Network.uploadFile',
    // request: 'Network.request',
    //用户接口
    toLogin: 'Interface.toLogin',
    checkLogin: 'Interface.checkLogin',
    getClientToken: 'Interface.getClientToken',
    getUserInfo: 'Interface.getUserInfo',
    ready: 'Interface.ready'
  };
  //覆写原型链，给继承者提供方法
  JOk.prototype = {
    constructor: JOk,
    init: function() {},
    config: function(opts) {
      if (opts.debug) {
        this.debug = true;
      }
      this.jsApiList = opts.jsApiList || [];
      if (this.jsApiList.length > 0) {
        for (var i = 0, len = this.jsApiList.length; i < len; i++) {
          this.nativeMethods.push(this.jsApiList[i].nativeName);
          if (typeof API[this.jsApiList[i].jsMethodName] === 'undefined') {
            this.createCustomMethod(
              this.jsApiList[i].jsMethodName,
              this.jsApiList[i].nativeName
            );
          }
        }
      }
    },
    ready: function(opts) {
      this.wrapInvoke('ready', opts);
    },
    error: function(opts) {
      this.wrapInvoke('error', opts);
    },
    wrapInvoke: function(interfaceNm, opts) {
      if (typeof opts === 'function') {
      } else {
        jlog(interfaceNm + ' need a function param');
        return;
      }
      var a = {};
      a.success = opts;
      invoke(interfaceNm, {}, a);
    },
    //扩展方法
    createCustomMethod: function(methodName, nativeName) {
      API[methodName] = nativeName;
      JOk.prototype[methodName] = function(opts) {
        checkInvoke(nativeName, parseObject(opts), opts);
      };
    },
    checkJsApi: function(opts) {
      invoke('checkJsApi', parseObject(opts), opts);
    },
    showToast: function(opts) {
      checkInvoke(API.showToast, parseObject(opts), opts);
    },
    showLoading: function(opts) {
      checkInvoke(API.showLoading, parseObject(opts), opts);
    },
    hideLoading: function(opts) {
      checkInvoke(API.hideLoading, parseObject(opts), opts);
    },
    showModal: function(opts) {
      checkInvoke(API.showModal, parseObject(opts), opts);
    },
    //界面 导航栏
    operateClientBottomBar: function(opts) {
      checkInvoke(API.operateClientBottomBar, parseObject(opts), opts);
    },
    // 获取发送邮件信息
    getEmailFileInfos: function(opts) {
      checkInvoke(API.getEmailFileInfos, parseObject(opts), opts);
    },
    // 跳转进入企业
    jumpToCorp: function(opts) {
      checkInvoke(API.jumpToCorp, parseObject(opts), opts);
    },
    // 消息页面打开同屏会议
    joinScreenShare: function(opts) {
      checkInvoke(API.joinScreenShare, parseObject(opts), opts);
    },
    // 显示vconsole
    showVconsole: function(opts) {
      checkInvoke(API.showVconsole, parseObject(opts), opts);
    },
    // 开通同屏进入同屏页面
    enterScreenShare: function(opts) {
      checkInvoke(API.enterScreenShare, parseObject(opts), opts);
    },
    // 调起客户端同屏支付
    payScreenShareOrder: function(opts) {
      checkInvoke(API.payScreenShareOrder, parseObject(opts), opts);
    },
    showShareMenu: function(opts) {
      checkInvoke(API.showShareMenu, parseObject(opts), opts);
    },
    hideShareMenu: function(opts) {
      checkInvoke(API.hideShareMenu, parseObject(opts), opts);
    },
    showNavigationBar: function(opts) {
      checkInvoke(API.showNavigationBar, parseObject(opts), opts);
    },
    hideNavigationBar: function(opts) {
      checkInvoke(API.hideNavigationBar, parseObject(opts), opts);
    },
    setNavigationBar: function(opts) {
      checkInvoke(API.setNavigationBar, parseObject(opts), opts);
    },
    closeWindow: function(opts) {
      checkInvoke(API.closeWindow, parseObject(opts), opts);
    },
    //设备与系统
    getSystemInfo: function(opts) {
      checkInvoke(API.getSystemInfo, parseObject(opts), opts);
    },
    getNetworkType: function(opts) {
      checkInvoke(API.getNetworkType, parseObject(opts), opts);
    },
    setClipboardData: function(opts) {
      checkInvoke(API.setClipboardData, parseObject(opts), opts);
    },
    getClipboardData: function(opts) {
      checkInvoke(API.getClipboardData, parseObject(opts), opts);
    },
    setScreenBrightness: function(opts) {
      checkInvoke(API.setScreenBrightness, parseObject(opts), opts);
    },
    getScreenBrightness: function(opts) {
      checkInvoke(API.getScreenBrightness, parseObject(opts), opts);
    },
    setKeepScreenOn: function(opts) {
      checkInvoke(API.setKeepScreenOn, parseObject(opts), opts);
    },
    vibrateLong: function(opts) {
      checkInvoke(API.vibrateLong, parseObject(opts), opts);
    },
    vibrateShort: function(opts) {
      checkInvoke(API.vibrateShort, parseObject(opts), opts);
    },
    scanCode: function(opts) {
      checkInvoke(API.scanCode, parseObject(opts), opts);
    },
    sendAction: function(opts) {
      checkInvoke(API.sendAction, parseObject(opts), opts);
    },
    //第三方
    launchMiniProgram: function(opts) {
      checkInvoke(API.launchMiniProgram, parseObject(opts), opts);
    },
    shareAppMessage: function(opts) {
      checkInvoke(API.shareAppMessage, parseObject(opts), opts);
    },
    shareMiniProgramMessage: function(opts) {
      checkInvoke(API.shareMiniProgramMessage, parseObject(opts), opts);
    },
    startPay: function(opts) {
      checkInvoke(API.startPay, parseObject(opts), opts);
    },
    //音频
    // startRecord: function(opts) {
    //   checkInvoke(API.startRecord, parseObject(opts), opts);
    // },
    // stopRecord: function(opts) {
    //   checkInvoke(API.stopRecord, parseObject(opts), opts);
    // },
    //位置
    getLocation: function(opts) {
      checkInvoke(API.getLocation, parseObject(opts), opts);
    },
    //数据缓存
    setStorage: function(opts) {
      checkInvoke(API.setStorage, parseObject(opts), opts);
    },
    getStorage: function(opts) {
      checkInvoke(API.getStorage, parseObject(opts), opts);
    },
    //媒体
    chooseImage: function(opts) {
      checkInvoke(API.chooseImage, parseObject(opts), opts);
    },
    //网络
    // downloadFile: function(opts) {
    //   checkInvoke(API.downloadFile, parseObject(opts), opts);
    // },
    // uploadFile: function(opts) {
    //   checkInvoke(API.uploadFile, parseObject(opts), opts);
    // },
    // request: function(opts) {
    //   checkInvoke(API.request, parseObject(opts), opts);
    // },
    //用户接口
    toLogin: function(opts) {
      checkInvoke(API.toLogin, parseObject(opts), opts);
    },
    checkLogin: function(opts) {
      checkInvoke(API.checkLogin, parseObject(opts), opts);
    },
    getClientToken: function(opts) {
      checkInvoke(API.getClientToken, parseObject(opts), opts);
    },
    getUserInfo: function(opts) {
      checkInvoke(API.getUserInfo, parseObject(opts), opts);
    }
  };

  var ok = new JOk(defaultParams);

  //兼容CommonJs规范
  if (typeof module !== 'undefined' && module.exports) module.exports = ok;

  //兼容AMD/CMD规范
  if (typeof define === 'function')
    define(function() {
      return ok;
    });

  //注册全局变量，兼容直接使用script标签引入该插件
  global.onekit = ok;

  //this，在浏览器环境指window，在nodejs环境指global
  //使用this而不直接用window/global是为了兼容浏览器端和服务端
  //将this传进函数体，使全局变量变为局部变量，可缩短函数访问全局变量的时间
})(this);
